// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntityRandomWalk.hpp"

using namespace Dawn;

EntityRandomWalk::EntityRandomWalk(SceneItem* item) : SceneItemComponent(item) {
  
}

std::vector<SceneItemComponent*> EntityRandomWalk::getDependencies() {
  return {
    (characterController = item->getComponent<CharacterController2D>())
  };
}

void EntityRandomWalk::onStart() {
  assertNotNull(characterController);

  useEvent([&](float_t delta) {
    // Do we need to pick a new destination
    if(timeLeftUntilNextDecision <= 0.0f) {
      // Pick a new destination to walk to.
      destination = randRange(-walkRange, walkRange);
      destination.x = mathAbs(destination.y) > mathAbs(destination.x) ? 0 : destination.x;
      destination.y = destination.x == 0 ? destination.y : 0;
      destination += glm::vec2(transform->getLocalPosition().x, transform->getLocalPosition().z);
      timeLeftUntilNextDecision = 4.0f;
    } else {
      timeLeftUntilNextDecision -= delta;
    }

    // Get the direction to move in
    glm::vec2 pos = destination - glm::vec2(
      transform->getLocalPosition().x,
      transform->getLocalPosition().z
    );

    // Conver to an angle and then convert back into a 2D vector. This mimics a
    // user essentially "moving the analogue stick in this direction"
    
    if(mathAbs(pos.x) > 1.0f || mathAbs(pos.y) > 1.0f) {
      float_t angle = atan2(pos.y, pos.x);
      glm::vec2 movement(cos(angle), sin(angle));
      transform->setLocalRotation(glm::quat(glm::vec3(0, -angle + 1.5708f, 0)));
      characterController->velocity += movement * delta * moveSpeed;
    }
    
  }, getScene()->eventSceneUpdate);
}